#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" A base class for downloading video metadata from the
    Youtube Data API v3 """


__all__ = [
    "YoutubeVideoMetadataDownloader"
]


import datetime
import dateutil

from .cache import (
    Cache
)
from .youtubecommentthreaddownloader import (
    YoutubeCommentThreadDownloader
)
from .youtubemetadatadownloader import (
    YoutubeMetadataDownloader
)
from .youtubevideo import (
    YoutubeVideo
)


class YoutubeVideoMetadataDownloader(YoutubeMetadataDownloader):
    """ TODO
    """
    ENDPOINT = "search"
    DATE_FORMAT = "{:%Y-%m-%dT%H:%M:%S.000Z}"

    def __init__(
            self,
            *args,
            **kwargs
    ):
        super().__init__(*args, **kwargs)

        try:
            self.search_terms = self._config["search_terms"]
        except (KeyError, AttributeError):
            self.search_terms = None
        self._published = None

    @property
    def _published(self):
        if self.__published is None:
            try:
                with Cache() as cache:
                    published = cache[self.search_terms]["publishedBefore"]
                    if isinstance(published, datetime.datetime):
                        published = \
                            self.DATE_FORMAT.format(published)
                    self.__published = published
            except KeyError:
                pass
        return self.__published

    @_published.setter
    def _published(self, published):
        self.__published = published

        if published is not None:
            try:
                published = dateutil.parser.isoparse(published)
                published += datetime.timedelta(seconds=1)

                with Cache() as cache:
                    if self.search_terms not in cache:
                        cache[self.search_terms] = {}

                    cache[self.search_terms]["publishedBefore"] = \
                        self.DATE_FORMAT.format(published)
            except ValueError:  # dateutil parser
                pass

    @property
    def _params(self):
        params = {
            "key": self._config["youtube_api_key"],
            "part": "snippet",
            "maxResults": 50,
            "order": "date",
            "type": "video",
            "q": self.search_terms
        }
        if self._published is not None:
            params["publishedBefore"] = self._published
        return params

    def download(self, search_terms=None):
        """ TODO """
        if search_terms is not None:
            self.search_terms = search_terms

        if self.search_terms is None:
            raise AttributeError(
                f"{self.__class__.__name__}.search_terms is undefined"
            )

        for item in self.items:
            video = YoutubeVideo(item)
            YoutubeCommentThreadDownloader(video.orig_id).download()
            del video

            self._published = item["snippet"]["publishedAt"]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" A base class for downloading comment threads and comments from the
    Youtube Data API v3 """


__all__ = [
    "YoutubeCommentThreadDownloader"
]


from .youtubemetadatadownloader import (
    YoutubeMetadataDownloader
)
from .youtubecommentthread import (
    YoutubeCommentThread
)


class YoutubeCommentThreadDownloader(YoutubeMetadataDownloader):
    """ TODO
    """
    ENDPOINT = "commentThreads"

    def __init__(
            self,
            video_id,
            *args,
            **kwargs
    ):
        super().__init__(*args, **kwargs)

        self.video_id = video_id
        self._next_page_token = None

    @property
    def _params(self):
        params = {
            "key": self._config["youtube_api_key"],
            "part": "snippet",
            "maxResults": 100,
            "textFormat": "plaintext",
            "videoId": self.video_id
        }
        if self._next_page_token is not None:
            params["pageToken"] = self._next_page_token
        return params

    def download(self, video_id=None):
        """ TODO """
        if video_id is not None:
            self.video_id = video_id

        if self.video_id is None:
            raise AttributeError(
                f"{self.__class__.__name__}.video_id is undefined"
            )

        for item in self.items:
            _ = YoutubeCommentThread(item)
            del _



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" A base class for self-aware database objects """


__all__ = [
    "DatabaseObject"
]


import psycopg2
import psycopg2.errors
import psycopg2.extensions
import psycopg2.extras

from .config import (
    Config
)
from .contextawarethreadedconnectionpool import (
    ContextAwareThreadedConnectionPool
)


class DatabaseObject:
    """ Base class for “self-aware” database objects such as e.g. social
        media posts. Instances share a common (class-level) psycopg2
        connectionPool, and a common config dict (currently only
        `config["connectionString"]` is used).

        Attributes:
            _connection_pool (private, class-wide):
                holds a psycopg2.pool.ConnectionPool instance which is shared
                across all class instances
            _config (private, class-wide):
                configuration dictionary, currently only `connectionString` is
                in use

        Args:
            config (dict): configuration dictionary.
                Specify `config["connectionString"]`, has to be set
                before first call to `connection_pool()`
    """
    _connection_pool = None
    _config = None

    def __init__(self, config={}):
        super().__init__()
        self.__class__._config = Config(config)

        self._id = None
        self._name = None

        try:
            # try to access table (creates tables if they don’t exist)
            self.execute_sql(
                psycopg2.sql.SQL(
                    """
                        SELECT
                            1
                        FROM
                            {table}
                        LIMIT 1;
                    """
                ).format(
                    table=psycopg2.sql.Identifier(self._table)
                )
            )
        except AttributeError:
            pass

    def __eq__(self, other):
        try:
            own_hash = self.__hash__()
        except AttributeError:
            own_hash = hash(self)
        try:
            other_hash = other.__hash__()
        except AttributeError:
            other_hash = hash(other)
        return own_hash == other_hash

    def __ne__(self, other):
        return not self.__eq__(other)

    def __str__(self):
        return self._name

    def __int__(self):
        try:
            _int = int(self._id)
        except TypeError:
            _int = -9999
        return _int

    def __repr__(self):
        _repr = f"<{self.__class__.__name__} ({self._name}: {self._id})>"
        return _repr

    @classmethod
    def connection_pool(cls):
        """ Returns a psycopg2.pool.ConnectionPool() object
            which is shared between instances (it’s created upon
            first call)

            Args:
                (none)
            Returns:
                :obj:`psycopg2.pool.ThreadedConnectionPool`
        """
        if cls._connection_pool is None:
            try:
                cls._connection_pool = \
                    ContextAwareThreadedConnectionPool(
                        0,
                        200,
                        cls._config["connection_string"]
                    )
            except AttributeError:
                raise AttributeError("No connection string defined")
        return cls._connection_pool

    def execute_sql(self, sql, sql_vars=(), commit=None):
        """ Runs the given SQL query

            Convenience function around getting a connection from
            connection_pool, creating a cursor and fetching results

            Args:
                sql (str): SQL query
                sql_vars (list or dict): variables to pass to
                    psycopg2.cursor().execute()

            Returns:
                mixed: results of SQL query as returned by
                    psycopg2.cursor().fetchall()
        """

        if commit is None:
            # if not specified whether query should be committed,
            # guess from the sql command

            # default: commit (safe choice, plus there’s so many more
            #          writing SQL commands to match than read-only)
            commit = True
            try:
                if sql.strip().split()[0].upper() in ("SELECT", "FETCH"):
                    commit = False
            except (KeyError, AttributeError, psycopg2.Error):
                pass

        with self.connection_pool().connection() as connection:
            with connection.cursor(
                    cursor_factory=psycopg2.extras.DictCursor
            ) as cursor:
                try:
                    cursor.execute(sql, sql_vars)
                except psycopg2.errors.UndefinedTable:
                    connection.rollback()
                    self._create_tables()
                    cursor.execute(sql, sql_vars)
                try:
                    results = cursor.fetchall()
                except psycopg2.ProgrammingError as e:
                    if str(e) == "no results to fetch":
                        results = None
                    else:
                        raise e

            if commit:
                connection.commit()

        return results

    def _create_tables(self):
        raise NotImplementedError(
            "Define `self._create_tables()` to create required tables."
        ) from None

    def adapt(self):
        """ adapts self’s integer representation to be used in DB """
        return psycopg2.extensions.adapt(self.id)


psycopg2.extensions.register_adapter(
    DatabaseObject,
    DatabaseObject.adapt
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" A class to save a Youtube user to the database and return its ID """


__all__ = [
    "YoutubeUser"
]


from .databaseobject import (
    DatabaseObject
)


class YoutubeUser(DatabaseObject):
    """ A class to save a Youtube user to
        the database and return its ID
    """

    def __init__(
            self,
            username,
            profile_url,
            *args,
            **kwargs
    ):
        super().__init__(*args, **kwargs)

        self._name = username
        self._id = self._save_to_database(username, profile_url)

    def _save_to_database(self, username, profile_url):
        return self.execute_sql(
            """
                INSERT INTO
                    users (
                        name,
                        profile_url
                    )
                    values (
                        %(name)s,
                        %(profile_url)s
                    )
                ON CONFLICT (profile_url)
                    DO UPDATE
                        set profile_url = EXCLUDED.profile_url
                RETURNING id;
            """,
            {
                "name": username,
                "profile_url": profile_url
            }
        )[0]["id"]

    def _create_tables(self):
        self.execute_sql(
            """
                CREATE TABLE IF NOT EXISTS
                    users (
                        id SERIAL PRIMARY KEY,
                        name TEXT,
                        profile_url TEXT UNIQUE
                    );
            """
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" A base class for downloading metadata from the
    Youtube Data API v3 """


__all__ = [
    "YoutubeMetadataDownloader"
]


import collections
import datetime
import json
import sys
import time

import requests

from .config import (
    Config
)


class YoutubeMetadataDownloader:
    """ Base class

        TODO
    """
    API_BASE_URL = "https://www.googleapis.com/youtube/v3/{endpoint:s}"
    PST = datetime.timezone(datetime.timedelta(hours=-8))
    THUMBNAIL_BASE_URL = \
        "https://img.youtube.com/vi/{video_id:s}/maxresdefault.jpg"

    ENDPOINT = None  # override in inheriting classes

    def __init__(self, config={}):
        self._config = Config(config)
        self.search_terms = None
        self._next_page_token = None

    @property
    def _params(self):
        raise NotImplementedError("Override self._params in child class")

    @property
    def items(self):
        """ TODO
        """
        while True:
            # DEBUG
            print(
                (
                    self.API_BASE_URL.format(endpoint=self.ENDPOINT),
                    self._params
                ),
                file=sys.stderr
            )
            with requests.get(
                    self.API_BASE_URL.format(endpoint=self.ENDPOINT),
                    params=self._params
            ) as response:
                data = response.json()

                if not response.ok:
                    if (
                            data["error"]["errors"][0]["reason"]
                            in ("dailyLimitExceeded", "quotaExceeded")
                    ):
                        now_pst = datetime.datetime.now(self.PST)
                        midnight_pst = datetime.datetime(
                            year=now_pst.year,
                            month=now_pst.month,
                            day=now_pst.day + 1,
                            tzinfo=self.PST
                        )
                        wait_time_until_quota_renewal = \
                            (midnight_pst - now_pst).total_seconds()

                        print(
                            (
                                "Quota reached, " +
                                "waiting until {:%Y-%m-%d %H:%M}"
                            ).format(
                                midnight_pst.astimezone()
                            ),
                            file=sys.stderr
                        )

                        time.sleep(wait_time_until_quota_renewal + (2 * 60))
                        continue
                    elif (
                            data["error"]["errors"][0]["reason"]
                            in ("commentsDisabled",)
                    ):
                        break

                    elif (
                            data["error"]["errors"][0]["reason"]
                            in ("processingFailure", "backendError")
                    ):
                        print(
                            "API error: processing failure/backend error",
                            file=sys.stderr
                        )

                    else:
                        raise ConnectionError(
                            json.dumps(
                                data,
                                sort_keys=True,
                                indent=4
                            )
                        )

                if (
                        "items" in data
                        and isinstance(data["items"], collections.Iterable)
                        and len(data["items"]) > 1
                ):
                    # testing for len() > 1, because it seems that
                    # publishedBefore is sometimes inclusive, sometimes
                    # exclusive the filter value -> loop at “beginning of time”
                    for item in data["items"]:
                        yield item
                else:
                    # print("No data returned", file=sys.stderr)
                    break

                try:
                    self._next_page_token = data["nextPageToken"]
                except KeyError:
                    # print("Reached last page", file=sys.stderr)
                    self._next_page_token = None
                    break

        # finally, get out of the generator ;)
        return  # ~ raise StopIteration()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" A class to save a Youtube a comment thread to the database and
    return its ID """


__all__ = [
    "YoutubeCommentThread"
]


import psycopg2

from .databaseobject import (
    DatabaseObject
)
from .youtubecomment import (
    YoutubeComment
)


# # EXAMPLE record
# {
#    "kind": "youtube#commentThread",
#    "etag": "\"SJZWTG6xR0eGuCOh2bX6w3s4F94/N-J1KPYA9h-RBd85_HSEeX3k8sw\"",
#    "id": "Ugw1hpNW--cMCwhG-wJ4AaABAg",
#    "replies": {
#     "comments": [
#      {
#       "kind": "youtube#comment",
#       "etag": "\"SJZWTG6xR0eGuCOh2bX6w3s4F94/uywljaTLRNTc5Xucl-OcyIF5dxs\"",
#       "id": "Ugw1hpNW--cMCwhG-wJ4AaABAg.90k0KongL2A93CeG-oebDs",
#       "snippet": {
#        "authorDisplayName": "Joko Prabowo",
#        "authorProfileImageUrl": "https://yt3.ggpht.com/...no-rj-mo",
#        "authorChannelUrl": "http://www.youtube.com/channel/UCZzay...",
#        "authorChannelId": {
#         "value": "UCZzaynt0Q3s3rEYEvWnCFKg"
#        },
#        "videoId": "wJitjtID0cc",
#        "textDisplay": "@Sugyono Saja betul om",
#        "textOriginal": "@Sugyono Saja betul om",
#        "parentId": "Ugw1hpNW--cMCwhG-wJ4AaABAg",
#        "canRate": true,
#        "viewerRating": "none",
#        "likeCount": 0,
#        "publishedAt": "2019-12-31T12:23:28.000Z",
#        "updatedAt": "2019-12-31T12:23:28.000Z"
#       }
#      },
#      {
#       "kind": "youtube#comment",
#       "etag": "\"SJZWTG6xR0eGuCOh2bX6w3s4F94/EwwgqCUTedRz4UQ0YrMDealtuec\"",
#       "id": "Ugw1hpNW--cMCwhG-wJ4AaABAg.90k0KongL2A90sDm4BCnYb",
#       "snippet": {
#        "authorDisplayName": "Sugyono Saja",
#        "authorProfileImageUrl": "https://yt3.ggpht.com/a/AGF-l7_eYs...j-mo",
#        "authorChannelUrl": "http://www.youtube.com/channel/UCVMHlmXv...",
#        "authorChannelId": {
#         "value": "UCVMHlmXvrgh-SASkqFy5nPw"
#        },
#        "videoId": "wJitjtID0cc",
#        "textDisplay": "Bikin burung laen drop. Suara kasar dan kerras",
#        "textOriginal": "Bikin burung laen drop. Suara kasar dan kerras",
#        "parentId": "Ugw1hpNW--cMCwhG-wJ4AaABAg",
#        "canRate": true,
#        "viewerRating": "none",
#        "likeCount": 1,
#        "publishedAt": "2019-11-03T11:36:28.000Z",
#        "updatedAt": "2019-11-03T11:36:28.000Z"
#       }
#      }
#     ]
#    }
#   }


class YoutubeCommentThread(DatabaseObject):
    """ A class to save a Youtube comment thread to
        the database and return its ID
    """

    def __init__(
            self,
            commentthread,
            *args,
            **kwargs
    ):
        super().__init__(*args, **kwargs)

        self._parse_commentthread_snippet(
            commentthread
        )

    def _get_video_id(self, orig_id):
        try:
            video_id = self.execute_sql(
                """
                    SELECT
                        id
                    FROM
                        videos
                    WHERE
                        orig_id = %(orig_id)s;
                """,
                {
                    "orig_id": orig_id
                }
            )[0]["id"]
        except (KeyError, psycopg2.Error):
            video_id = None
        return video_id

    def _parse_commentthread_snippet(
            self,
            commentthread
    ):
        orig_id = commentthread["id"]
        video_id = self._get_video_id(
            commentthread["snippet"]["videoId"]
        )

        self._id = self._save_to_database(orig_id, video_id)
        self._name = f"{orig_id[:10]}..."

        # parse comments
        order_in_thread = 0

        try:
            _ = YoutubeComment(
                commentthread["snippet"]["topLevelComment"],
                self._id,
                video_id,
                order_in_thread
            )
            del _
            order_in_thread += 1
        except KeyError as e:
            if e.args[0] in ("snippet", "topLevelComment"):
                pass
            else:
                raise e

        try:
            for comment in commentthread["replies"]["comments"]:
                _ = YoutubeComment(
                    comment,
                    self._id,
                    video_id,
                    order_in_thread
                )
                del _
                order_in_thread += 1
        except KeyError as e:
            if e.args[0] in ("replies",):
                pass
            else:
                raise e

    def _save_to_database(self, orig_id, video_id):
        return self.execute_sql(
            """
                INSERT INTO
                    comment_threads (
                        orig_id,
                        video_id
                    )
                    values (
                        %(orig_id)s,
                        %(video_id)s
                    )
                ON CONFLICT (orig_id)
                    DO UPDATE
                        set video_id = EXCLUDED.video_id
                RETURNING id;
            """,
            {
                "orig_id": orig_id,
                "video_id": video_id
            }
        )[0]["id"]

    def _create_tables(self):
        self.execute_sql(
            """
                CREATE TABLE
                    comment_threads (
                        id SERIAL PRIMARY KEY,
                        orig_id CHAR(26) NOT NULL UNIQUE,
                        video_id INTEGER REFERENCES videos(id)
                    );

                CREATE INDEX IF NOT EXISTS
                    comment_threads_video_id_idx
                ON
                    comment_threads
                        (video_id);
            """
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" Imports global configuration from a default location """


__all__ = ["Config"]


import argparse
import os
import os.path
import warnings

import yaml


class Config:
    """ Global configuration object

        Tries to load configuration from YAML files in default locations
        (/etc/{module}.yml, ~/.config/{module}.yml, %APPDATA%/{module}.yml,
        ${XDG_CONFIG_HOME}/{module}.yml).

        Configuration file location can be overriden by specifying
        `config_files` or a base name `config_filesBaseName` (replaces
        “{module}” in the file locations listed above)

        Arguments:
            content of the config file

        Args:
            config_files (list of str, optional): override configuration
                file locations
            config_files_basename (str): override configuration file names
    """
    DEFAULT_CONFIG = os.path.join(
        os.path.dirname(__file__),
        "default.yml"
    )

    def __init__(
            self,
            config={},
            config_files=None,
            config_files_basename=None
    ):
        self._config = {}
        self._config_files = []

        if config_files is not None:
            if not isinstance(config_files, (list, tuple)):
                config_files = [config_files]
            self._config_files = config_files

        elif config_files_basename is None:
            config_files_basename = self.__module__.split(".")[0]

            self._config_files = [
                "/etc/{:s}.yml".format(config_files_basename),
                os.path.abspath(
                    os.path.join(
                        (
                            os.environ.get('APPDATA') or
                            os.environ.get('XDG_CONFIG_HOME') or
                            os.path.join(os.environ['HOME'], '.config')
                        ),
                        "{:s}.yml".format(config_files_basename)
                    )
                )
            ]

        self._config = self._load_config()
        self._config.update(
            self._parse_arguments()
        )
        self._config.update(config)

    def _load_config(self):
        config = {}

        for config_file in self._config_files:
            try:
                config.update(
                    yaml.safe_load(
                        open(config_file, "r", encoding="utf-8")
                    )
                )
            except FileNotFoundError:
                pass

        if config == {}:
            warnings.warn(
                "No configuration found in files {}".format(
                    ",".join(self._config_files)
                )
            )

        return config

    @staticmethod
    def _parse_arguments():
        # command line arguments override
        # config file options
        argparser = argparse.ArgumentParser()

        argparser.add_argument(
            "-p",
            "--postgresql-connection-string",
            help="""Store the retrieved data in this PostgreSQL data base"""
        )

        argparser.add_argument(
            "-a",
            "--youtube-api-key",
            help="""Use this API key for the YouTube Data API v3"""
        )

        argparser.add_argument(
            "search_terms",
            help="""Query the YouTube API for these search terms"""
        )

        args = argparser.parse_args()

        _config = {}
        if args.postgresql_connection_string is not None:
            _config["connection_string"] = args.postgresql_connection_string
        if args.youtube_api_key is not None:
            _config["youtube_api_key"] = args.youtube_api_key
        if args.search_terms is not None:
            _config["search_terms"] = args.search_terms

        return _config

    def __getitem__(self, pos):
        return self._config[pos]

    def __setitem__(self, pos, value):
        self._config[pos] = value

    def __iter__(self):
        return iter(self._config)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" A class to save a Youtube video to the database and return its ID """


__all__ = [
    "YoutubeVideo"
]


import dateparser

from .databaseobject import (
    DatabaseObject
)
from .youtubechannel import (
    YoutubeChannel
)


#  # EXAMPLE record
#  {
#   "kind": "youtube#searchResult",
#   "etag": "\"SJZWTG6xR0eGuCOh2bX6w3s4F94/XUIrQ-4pn8OEdWxKzr87gUo0L3U\"",
#   "id": {
#    "kind": "youtube#video",
#    "videoId": "dGqZg2eeFPw"
#   },
#   "snippet": {
#    "publishedAt": "2018-12-26T08:56:17.000Z",
#    "channelId": "UC3YsVCnrtanUqE02uE7Akyw",
#    "title": "Masteran Poksai mandarin,Ngeplong vol keras",
#    "description": "Masteran poksai mandarin langsung nyaut.",
#    "thumbnails": {
#     "default": {
#      "url": "https://i.ytimg.com/vi/dGqZg2eeFPw/default.jpg",
#      "width": 120,
#      "height": 90
#     },
#     "medium": {
#      "url": "https://i.ytimg.com/vi/dGqZg2eeFPw/mqdefault.jpg",
#      "width": 320,
#      "height": 180
#     },
#     "high": {
#      "url": "https://i.ytimg.com/vi/dGqZg2eeFPw/hqdefault.jpg",
#      "width": 480,
#      "height": 360
#     }
#    },
#    "channelTitle": "KICAU NUSANTARA",
#    "liveBroadcastContent": "none"
#   }
#  }

class YoutubeVideo(DatabaseObject):
    """ A class to save a Youtube video to
        the database and return its ID
    """

    def __init__(
            self,
            video,
            *args,
            **kwargs
    ):
        super().__init__(*args, **kwargs)

        self._name = None
        self._id = None
        self.orig_id = None

        self._parse_video_snippet(video)

    def _parse_video_snippet(self, video):
        orig_id = video["id"]["videoId"]
        channel_id = int(
            YoutubeChannel(
                video["snippet"]["channelId"],
                video["snippet"]["channelTitle"]
            )
        )
        published = dateparser.parse(
            video["snippet"]["publishedAt"]
        )
        title = video["snippet"]["title"]
        description = video["snippet"]["description"]

        self._id = self._save_to_database(
            orig_id,
            channel_id,
            published,
            title,
            description
        )
        self._name = title
        self.orig_id = orig_id

    def _save_to_database(
            self,
            orig_id,
            channel_id,
            published,
            title,
            description
    ):
        return self.execute_sql(
            """
                INSERT INTO
                    videos (
                        orig_id,
                        channel_id,
                        published,
                        title,
                        description
                    )
                    values (
                        %(orig_id)s,
                        %(channel_id)s,
                        %(published)s,
                        %(title)s,
                        %(description)s
                    )
                ON CONFLICT (orig_id)
                    DO UPDATE
                        SET
                            channel_id = EXCLUDED.channel_id,
                            published = EXCLUDED.published,
                            title = EXCLUDED.title,
                            description = EXCLUDED.description
                RETURNING id;
            """,
            {
                "orig_id": orig_id,
                "channel_id": channel_id,
                "published": published,
                "title": title,
                "description": description
            }
        )[0]["id"]

    def _create_tables(self):
        self.execute_sql(
            """
                CREATE TABLE IF NOT EXISTS
                    videos (
                        id SERIAL PRIMARY KEY,
                        orig_id CHAR(11) UNIQUE,
                        channel_id INTEGER REFERENCES channels(id),
                        published TIMESTAMP,
                        title TEXT,
                        description TEXT
                    );
                CREATE INDEX IF NOT EXISTS
                    videos_channel_id_idx
                ON
                    videos
                        (channel_id);

                CREATE INDEX IF NOT EXISTS
                    videos_published_idx
                ON
                    videos
                        (published);
            """
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.


""" A psycopg2.ThreadedConnectionPool providing a `connection()` context. """


import contextlib

import psycopg2.pool


class ContextAwareThreadedConnectionPool(psycopg2.pool.ThreadedConnectionPool):
    """ ContextAwareThreadedConnectionPool

    A psycopg2.ThreadedConnectionPool providing a `connection()` context

    Example::

        connectionPool = ContextAwareThreadedConnectionPool(
            minconn=0,
            maxconn=100,
            "dbname=foobar"
        )
        with connectionPool.connection() as connection:
            with connection.cursor() as cursor:
                cursor.execute(
                    '''
                        SELECT
                            field1
                        FROM
                            table1;
                    '''
                )
                result = cursor.fetchall()
        print(result)

    """
    @contextlib.contextmanager
    def connection(self):
        """
            returns a psycopg2.connection object,
            can be used as a context manager
        """
        connection = self.getconn()
        try:
            yield connection
        finally:
            self.putconn(connection)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

__all__ = [
    "__name__",
    "__version__",
    "YoutubeVideoMetadataDownloader"
]

from .lib.youtubevideometadatadownloader import (
    YoutubeVideoMetadataDownloader
)

__name__ = "metatube"

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" A class to save a Youtube comment to the database and return its ID """


__all__ = [
    "YoutubeComment"
]


import dateparser

from .databaseobject import (
    DatabaseObject
)
from .youtubeuser import (
    YoutubeUser
)


#  # EXAMPLE record
#     {
#      "kind": "youtube#comment",
#      "etag": "\"SJZWTG6xR0eGuCOh2bX6w3s4F94/EwwgqCUTedRz4UQ0YrMDealtuec\"",
#      "id": "Ugw1hpNW--cMCwhG-wJ4AaABAg.90k0KongL2A90sDm4BCnYb",
#      "snippet": {
#       "authorDisplayName": "Sugyono Saja",
#       "authorProfileImageUrl": "https://yt3.ggpht.com/a...-mo",
#       "authorChannelUrl": "http://www.you...Fy5nPw",
#       "authorChannelId": {
#        "value": "UCVMHlmXvrgh-SASkqFy5nPw"
#       },
#       "videoId": "wJitjtID0cc",
#       "textDisplay": "Bikin burung laen drop. Suara kasar dan kerras",
#       "textOriginal": "Bikin burung laen drop. Suara kasar dan kerras",
#       "parentId": "Ugw1hpNW--cMCwhG-wJ4AaABAg",
#       "canRate": true,
#       "viewerRating": "none",
#       "likeCount": 1,
#       "publishedAt": "2019-11-03T11:36:28.000Z",
#       "updatedAt": "2019-11-03T11:36:28.000Z"
#      }

class YoutubeComment(DatabaseObject):
    """ A class to save a Youtube comment to
        the database and return its ID
    """

    def __init__(
            self,
            comment,
            comment_thread_id,
            video_id,
            order_in_thread,
            *args,
            **kwargs
    ):
        super().__init__(*args, **kwargs)

        self._parse_comment_snippet(
            comment,
            comment_thread_id,
            video_id,
            order_in_thread
        )

    def _parse_comment_snippet(
            self,
            comment,
            comment_thread_id,
            video_id,
            order_in_thread
    ):
        text = comment["snippet"]["textOriginal"]
        published = dateparser.parse(
            comment["snippet"]["publishedAt"]
        )
        updated = dateparser.parse(
            comment["snippet"]["updatedAt"]
        )
        author_id = int(
            YoutubeUser(
                comment["snippet"]["authorDisplayName"],
                comment["snippet"]["authorChannelUrl"]
            )
        )

        self._id = self._save_to_database(
            published,
            updated,
            author_id,
            comment_thread_id,
            video_id,
            order_in_thread,
            text
        )
        self._name = f"{text[:10]}..."

    def _save_to_database(
            self,
            published,
            updated,
            author_id,
            comment_thread_id,
            video_id,
            order_in_thread,
            text
    ):
        return self.execute_sql(
            """
                INSERT INTO
                    comments (
                        published,
                        updated,
                        author_id,
                        comment_thread_id,
                        video_id,
                        order_in_thread,
                        text
                    )
                    values (
                        %(published)s,
                        %(updated)s,
                        %(author_id)s,
                        %(comment_thread_id)s,
                        %(video_id)s,
                        %(order_in_thread)s,
                        %(text)s
                    )
                ON CONFLICT (comment_thread_id, order_in_thread)
                    DO UPDATE
                        SET
                            published = EXCLUDED.published,
                            updated = EXCLUDED.updated,
                            author_id = EXCLUDED.author_id,
                            video_id = EXCLUDED.video_id,
                            text = EXCLUDED.text
                RETURNING id;
            """,
            {
                "published": published,
                "updated": updated,
                "author_id": author_id,
                "comment_thread_id": comment_thread_id,
                "video_id": video_id,
                "order_in_thread": order_in_thread,
                "text": text
            }
        )[0]["id"]

    def _create_tables(self):
        self.execute_sql(
            """
                CREATE TABLE
                    comments (
                        id SERIAL PRIMARY KEY,
                        published TIMESTAMP,
                        updated TIMESTAMP,
                        author_id INTEGER NOT NULL REFERENCES users(id),
                        comment_thread_id INTEGER NOT NULL
                            REFERENCES comment_threads(id),
                        video_id INTEGER NOT NULL REFERENCES videos(id),
                        order_in_thread INTEGER NOT NULL DEFAULT 0,
                        text TEXT,
                        UNIQUE(comment_thread_id, order_in_thread)
                    );

                CREATE INDEX IF NOT EXISTS
                    comments_published_idx
                ON
                    comments
                        (published);

                CREATE INDEX IF NOT EXISTS
                    comments_author_id_idx
                ON
                    comments
                        (author_id);

                CREATE INDEX IF NOT EXISTS
                    comments_comment_thread_id_idx
                ON
                    comments
                        (comment_thread_id);

                CREATE INDEX IF NOT EXISTS
                    comments_video_id_idx
                ON
                    comments
                        (video_id);

                CREATE INDEX IF NOT EXISTS
                    comments_first_in_thread_idx
                ON
                    comments
                        ((order_in_thread = 0));
            """
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import setuptools
import versioneer


with open("README.md") as f:
    LONG_DESCRIPTION = f.read()

with open("requirements.txt") as f:
    REQUIREMENTS = f.read()

PACKAGE_NAMES = setuptools.find_packages()

print(setuptools.find_packages())

setuptools.setup(
    name=PACKAGE_NAMES[0],
    version=versioneer.get_version(),
    author="Christoph Fink",
    author_email="christoph.fink@helsinki.fi",
    description="""Download YouTube metadata for videos
        relating to a search query""",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/helics-lab/metatube",
    packages=PACKAGE_NAMES,
    install_requires=REQUIREMENTS,
    cmdclass=versioneer.get_cmdclass(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent"
    ],
    license="GPLv3",
    entry_points={
        "console_scripts": [
            "metatube=metatube.bin.metatube:main"
        ]
    }

)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright (C) 2019 Christoph Fink, University of Helsinki
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 3
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

""" A class to save a Youtube channel to the database and return its ID """


__all__ = [
    "YoutubeChannel"
]


from .databaseobject import (
    DatabaseObject
)


class YoutubeChannel(DatabaseObject):
    """ A class to save a Youtube channel to
        the database and return its ID
    """

    def __init__(
            self,
            orig_id,
            title,
            *args,
            **kwargs
    ):
        super().__init__(*args, **kwargs)

        self._name = title
        self._orig_id = orig_id
        self._id = self._save_to_database(orig_id, title)

    def _save_to_database(self, orig_id, title):
        return self.execute_sql(
            """
                INSERT INTO
                    channels (
                        orig_id,
                        title
                    )
                    values (
                        %(orig_id)s,
                        %(title)s
                    )
                ON CONFLICT (orig_id)
                    DO UPDATE
                        set title = EXCLUDED.title
                RETURNING id;
            """,
            {
                "orig_id": orig_id,
                "title": title
            }
        )[0]["id"]

    def _create_tables(self):
        self.execute_sql(
            """
                CREATE TABLE IF NOT EXISTS
                    channels (
                        id SERIAL PRIMARY KEY,
                        orig_id CHAR(24) UNIQUE,
                        title TEXT
                    );
            """
        )
